@@ -76,27 +76,27 @@ class TwitterAPI(object):
 
         :returns: TwitterAPI.TwitterResponse object
         """
+        resource, endpoint = self._get_endpoint(resource)        
+        if endpoint not in ENDPOINTS:
+            raise Exception('Endpoint "%s" unsupported' % endpoint)
         session = requests.Session()
         session.auth = self.auth
         session.headers = {'User-Agent': USER_AGENT}
-        resource, endpoint = self._get_endpoint(resource)
-        if endpoint in STREAMING_ENDPOINTS:
-            session.stream = True
-            method = 'GET' if params is None else 'POST'
-            url = self._prepare_url(STREAMING_ENDPOINTS[endpoint][0], resource)
-            timeout = STREAMING_SOCKET_TIMEOUT
-        elif endpoint in REST_ENDPOINTS:
-            session.stream = False
-            method = REST_ENDPOINTS[endpoint][0]
-            url = self._prepare_url(REST_SUBDOMAIN, resource)
-            timeout = REST_SOCKET_TIMEOUT
-        else:
-            raise Exception('"%s" is not valid endpoint' % resource)
+        method, subdomain = ENDPOINTS[endpoint]
+        url = self._prepare_url(subdomain, resource)
         if method is 'POST':
             data = params
             params = None
         else:
             data = None
+        if 'stream' in subdomain:
+            session.stream = True
+            timeout = STREAMING_TIMEOUT
+            # always use 'delimited' for efficient stream parsing
+            params['delimited'] = 'length'
+        else:
+            session.stream = False
+            timeout = REST_TIMEOUT
         r = session.request(
             method,
             url,
@@ -134,6 +134,10 @@ class TwitterResponse(object):
     def text(self):
         """:returns: Raw API response text."""
         return self.response.text
+        
+    def json(self):
+        """:returns: response as JSON object."""
+        return self.response.json()
 
     def get_iterator(self):
         """:returns: Iterator for tweets or other message objects in response."""
@@ -146,7 +150,7 @@ class TwitterResponse(object):
         return self.get_iterator()
 
     def get_rest_quota(self):
-        """:returns: Quota information in the response header.  Valid only for REST API responses."""
+        """:returns: Quota information in the REST-only response header."""
         remaining, limit, reset = None, None, None
         if self.response:
             if 'x-rate-limit-remaining' in self.response.headers:
@@ -168,6 +172,7 @@ class _RestIterable(object):
 
     def __init__(self, response):
         resp = response.json()
+        # hack to force json response into something iterable
         if 'errors' in resp:
             self.results = resp['errors']
         elif 'statuses' in resp:
@@ -194,19 +199,38 @@ class _StreamingIterable(object):
     """
 
     def __init__(self, response):
-        self.results = response.iter_lines(1)
-
+        self.stream = response.raw
+    
+    def _iter_stream(self):
+        """Return next item in the stream (with or without 'delimited')."""
+        while True:
+            item = None
+            buf = bytearray()
+            while True:
+                # add bytes until item boundary reached
+                buf += self.stream.read(1)
+                if buf[-2:] == b'\r\n': 
+                    item = buf[0:-2]
+                    # when delimited=length, use byte size to read next item
+                    if item.isdigit():  
+                        item = self.stream.read(int(item))
+                    break
+            yield item
+    
     def __iter__(self):
         """Return a tweet status as a JSON object."""
-        for item in self.results:
+        for item in self._iter_stream():
             if item:
-                yield json.loads(item.decode('utf-8'))
+                try:
+                    yield json.loads(item.decode('utf-8'))
+                except ValueError:
+                	continue
 
 
 def RestIterator(*args, **kwargs):
     """Deprecated. Use _RestIterable instead."""
     from warnings import warn
-    warn('RestIterator is deprecated. Use _RestIterable instead',
+    warn('RestIterator is deprecated. Use _RestIterable instead.',
          DeprecationWarning,
          stacklevel=2)
     return _RestIterable(*args, **kwargs)
@@ -215,7 +239,7 @@ def RestIterator(*args, **kwargs):
 def StreamingIterator(*args, **kwargs):
     """Deprecated. Use _StreamingIterable instead."""
     from warnings import warn
-    warn('StreamingIterator is deprecated. Use _StreamingIterable instead',
+    warn('StreamingIterator is deprecated. Use _StreamingIterable instead.',
          DeprecationWarning,
          stacklevel=2)
     return _StreamingIterable(*args, **kwargs)
